
;**********************************************
;* DSReset                                    *
;* Sends a reset pulse to all devices.        *
;* Returns carry set if devices present.      *
;* Returns carry clear if no devices present. *
;**********************************************
DSReset:
        sbi    DDRC,PinDSBus      ;DDRB bit 2 = 1 = output
        cbi    PORTC,PinDSBus     ;set 1-wire bus low
        ldi    temp3,d250us
        rcall  Delay
        rcall  Delay           ;Wait 250us x 2 = 500 us
        cbi    DDRC,PinDSBus      ;Switch DS bus bit to input (pulled up high)
        sbi    PORTC,PinDSBus     ;turn on pullup
        ldi    temp3,d100us
        rcall  Delay           ;Wait 100 us for the DS1820 to respond
        clc                    ;exit with carry clear if no device
        sbis   PINC,PinDSBus      ;skip next if port B bit 2=1
        sec                    ;bit was clear (=0) so set carry flag
        ldi    temp3,d250us
        rcall  Delay
        rcall  Delay           ;Wait 500 us

			ret

;**********************************************
;* GetBit                                     *
;* Carry holds the bit from the DS1820 bus    *
;**********************************************
GetBit:
        ldi    temp3,d2us   ;wait for the 1820 data
        sbi    DDRC,PinDSBus      ;DDRB bit 2 = 1 = output
        cbi    PORTC,PinDSBus     ;set 1-wire bus low	
	     rcall  Delay		
        cbi    DDRC,PinDSBus      ;DDRB bit 2 = 0 = input
        sbi    PORTC,PinDSBus     ;turn on pullup
        ldi    temp3,d15us  ;wait 15us for the 1820 data
		  rcall  Delay
		  clc                    ;assume 1-Wire bus is low
        sbic   PINC,PinDSBus
        sec                    ;1-Wire bus is high, so set carry flag
        ldi    temp3,d44us
        rcall  Delay
        ret

;**********************************************
;* Write bit in CC to DS bus                  *
;**********************************************
PutBit:
        sbi    DDRC,PinDSBus      ;DDRB bit 2 = 1 = output
        cbi    PORTC,PinDSBus     ;Pull Port B, bit 2 low
        ldi    temp3,d15us  ;wait for the 1820 sample window
		  rcall  Delay
        brcc   Put0            ;if carry = 0, send a 0
        cbi    DDRC,PinDSBus      ;otherwise, send a 1
        sbi    PORTC,PinDSBus     ;turn on pullup
Put0:
        ldi    temp3,d44us  ;load 44us delay in delay register
		  rcall  Delay           ;and wait
        cbi    DDRC,PinDSBus      ;this signals the end of a write 0
        sbi    PORTC,PinDSBus     ;turn on pullup
        ldi    temp3,d2us   ;let the DS1820 rest & digest
        rcall  Delay
        ret

;**********************************************
;* GetByte                                    *
;* temp1 returns the byte from the DS1820 bus  *
;**********************************************
GetByte:
        ldi    temp2,8          ;bit count
        clr    temp1
NxtBit:

		  rcall  GetBit
        ror    temp1            ;shift carry bit into temp1
        dec    temp2
        brne   NxtBit          ;repeat for all 8 bits
        ret

;**********************************************
;* PutByte                                    *
;* data in temp1                               *
;**********************************************
PutByte:
        ldi    temp2,8          ;bit counter
ShiftB:
        lsr    temp1            ;shift bits in temp1 into carry

		  rcall  PutBit
NextB:
        dec    temp2
        brne   ShiftB
        ret

;**********************************************
;* Search for devices on DS bus               *
;* Based on Dallas Semiconductor's ML6805     *
;* code.                                      *
;**********************************************
ROM1st:
        ldi    temp1,$41        ;Point Rom Search to top (1st part).
        mov    LastDis,temp1
        rcall  ROMSrch
        ret

ROMSrch:
        push   ZL
        push   ZH

        rcall  DSReset         ;Reset 1-wire bus.
        brcs   r_look
        rjmp   NoDev
r_look:
        ldi    temp1,64        ;Prepare to find 64 bits of ROM data.
        mov    BitNum,temp1
        ldi    temp4,8        ;Initial value for eight counter.
        mov    LasDisCpy, LastDis ;Save copy of last dis. from last search.
        ldi    ZL,low(romdta)
        ldi    ZH,high(romdta) ;point to 1st byte of ROM data
        sts    romoff,ZL       ;buffer in rombuff.
        sts    romoff+1,ZH
        ldi    temp1,SearchROM  ;Get ROM search command in temp1.
        rcall  PutByte         ;Send ROM search command to DOW bus.

NxtRBit:
        clr    temp1
        clr    temp2
        rcall  GetBit          ;1st read time slot.
        rol    temp2            ;reg temp2, bit0 has first result
        rcall  GetBit          ;2nd read time slot.
        rol    temp1
        cpi    temp2,1          ;Was the first bit received 1?
        brne   no_or           ;Branch if it wasn't
        ori    temp1,2          ;Set bit 1 in temp1

no_or:
        cpi    temp1,3          ;See if error condition occurred.
        breq   NoDev           ;Abort if error.
        cpi    temp1,0          ;Was there a disagreement on bus.
        brne   nosave          ;Branch if there wasn't.
        cp     LasDisCpy,BitNum ;Compare last dis to current position.
        brne   lwhere          ;If its not equal look some more.
        ldi    temp1,1          ;We've been on this path and sent a 0.
        rjmp   wrtBit          ;now lets send a 1.

lwhere:
        brcs   sLstBit         ;Stay on previous path.
        clr    temp1            ;Past last dis. fall to the right.
        rjmp   savedis         ;Send 0 to bus.
sLstBit:
        lds    ZL,romoff       ;Get offset of ROM data buffer in
        lds    ZH,romoff+1
        ld     temp1,Z          ;index register, and byte in temp1.
        andi   temp1,1          ;Isolate lsb of temp1.
        cpi    temp1,1          ;See if it was a 1.
        breq   wrtBit          ;Don't save as dis position if so.
savedis:
        mov    LastDis,BitNum  ;Get current position in x register.
                               ;Save current last disagreement pos.
        rjmp   wrtBit          ;Send the write bit time slot.
nosave:
        ldi    temp2,1
        eor    temp1,temp2       ;Flip lsb of temp1.
wrtBit:
        andi   temp1,1          ;Isolate lsb of temp1
        mov    temp2,temp1
        lsr    temp2
        rcall  PutBit          ;Send lsb of temp1 to 1-wire bus.
        lds    ZL,romoff
        lds    ZH,romoff+1     ;Get offset into ROM data buffer.
        ld     temp1,Z          ;Get previous val of rombyte.
        ror    temp1            ;Get ROM bit in MSB of rombyte.
        st     Z,temp1          ;Store new byte value & point to next byte
        dec    temp4          ;Decrement byte counter.
        brne   bnfull          ;Branch if we don't have full byte.
        adiw   ZH:ZL,1         ;increment x
        sts    romoff,ZL       ;buffer in rombuff.
        sts    romoff+1,ZH
        ldi    temp4,8        ;reload the eight counter.
bnfull:
        dec    BitNum          ;Decrement 64 bit counter.
        brne   NxtRBit         ;Go get next rom bit.
        cp     LasDisCpy,LastDis ;Compare to last dis. copy
        brne   MoDows          ;If not equal clear temp1.
        ldi    temp1,$41        ;Point ROM search algorithm back
        mov    LastDis,temp1    ;to first part.
        ldi    temp1,1          ;Indicate this was last part on bus.
        rjmp   lastdow         ;Skip the clra statement and leave.
MoDows:
        clr    temp1            ;Indicate that more parts are on bus.
lastdow:
        sec                    ;Indicate success.
        rjmp   DevFnd          ;Skip clear carry stuff.
NoDev:
        clc                    ;Indicate failure.
DevFnd:
        pop    ZH
        pop    ZL
        ret

;**********************************************
;* GetROMS                                    *
;* Looks for up to 8 DS1820s, reads their     *
;* 64 bit ROM codes and saves them in memory  *
;**********************************************
GetROMS:

		  clr    temp1
        clr    ROMCount        ;clear out ROMCount
        clr    Done            ;clear Done flag
        ldi    YH,high(romdta)
        ldi    YL,low(romdta)  ;Y points to romdta buffer
ClrOut:
        st     Y+,temp1         ;clear out the ROM ID storage area
        cpi    YL,low(Sensor7+8)
        brcs   ClrOut
LookROM:
        rcall  DSReset         ;Reset 1-wire bus
        brcc   SetExit         ;Abort if no presence detect

        rcall  ROM1st          ;look for the first ROM
        sts    SSScratchL,temp1
        brcc   SetExit         ;if temp1 = 0 then there are more devices
        ldi    ZH,high(Sensor0)
        ldi    ZL,low(Sensor0) ;X points to first device ID
        rjmp   MR0             ;process them & exit
LR1:
        rcall  ROMSrch         ;look for other devices
        sts    SSScratchL,temp1
        breq   MR0             ;if temp1 = 0 then there are more devices
        inc    Done
MR0:

		  ldi    YH,high(romdta+8)
        ldi    YL,low(romdta+8) ;Y points to ROM just read

MoveROM:
        ld     temp1,-Y         ;move byte from romdta
        st     Z+,temp1         ;to SensorX
        cpi    YL,low(romdta)  ;move all 8 bytes
        brne   MoveROM         ;of the ROM ID
        inc    ROMCount        ;increment the ROM counter
        lds    temp3,SSScratchL
		  tst    temp3        ;repeat until the last device found
        breq   MR2
        inc    Done
MR2:
        tst    Done
        brne   SetExit
        cpi    ZL,low(Sensor7+8) ;or until the end of the ROM
        brcs   LR1             ;storage area is reached

SetExit:
        ret



;**********************************************
;* Delay   time in us based on CLK/8          *
;* temp3 holds the delay value.            *
;**********************************************
Delay:


		  in      SaveSREG,SREG
        push 	 temp1		
        ldi 	 temp1,0xFF
        out 	 TCNT1H,temp1
		  out     TCNT1L,temp3
        clt
        ldi     temp3,TCK8
        out     TCCR1B,temp3
		  in 	 	 temp1,TIMSK
        sbr 	 temp1,1<<TOIE1
        out 	 TIMSK,temp1

Dwait:
        brtc    Dwait
        pop		 temp1
		  out     SREG,SaveSREG
        ret


;****************************************************
ReadScratch:
 		
 		  rcall  DSReset         ;reset bus
 		
        rcall  ROMatch         ;select device again

        ldi    temp1,RdScratch  ;read scratchpad command
        rcall  PutByte
        ldi    ZL,low(TempLSB) ;point to Scratchpad storage
        ldi    ZH,high(TempLSB)


RdMem:                         ;read in the 1820 scratchpad memory
        	
		  rcall  GetByte         ;read result
        st     Z+,temp1         ;ScratchL byte
        cpi    ZL,low(TempLSB+9) ;read 9 bytes
        brcs   RdMem

		ret

;***************************************************
IndexToOffset:
        ldi    ZL,low(Sensor0)
        ldi    ZH,high(Sensor0)


        mov    temp2,temp1       ;copy device # to temp2
        lsl    temp2            ;temp2 x 2
        lsl    temp2            ;temp2 x 4
        lsl    temp2            ;temp2 x 8
        adc    ZL,temp2
        brcc   GT1
        inc    ZH              ;carry set, so add to XH and point to device
GT1:
        sts    romoff,ZL
        sts    romoff+1,ZH     ;used to point to Temp result
        ret


;**********************************************
;* ROMatch                                    *
;* Sends the MatchROM command, followed by    *
;* the 8 byte ROM ID                          *
;**********************************************
ROMatch:
        lds    ZL,romoff       ;point to device ROM
        lds    ZH,romoff+1
        adiw   ZH:ZL,8         ;point to MSB
        ldi    temp4,8
        ldi    temp1,MatchROM   ;Match ROM command
        rcall  PutByte         ;send it
SndROM:
        ld     temp1,-Z         ;send the ROM ID
        rcall  PutByte
        dec    temp4
        brne   SndROM
        ret
			



;**********************************************
;* GetTemp                                    *
;* Reads the extended Temperatures from the   *
;* DS1820/18S20 device selected in A.         *
;* Result stored in the temp1 storage area     *
;**********************************************
GetTemp:
		
		  sei
		  rcall	ClearInt
		  rcall  DSReset
        	
        rcall IndexToOffset
			
        rcall  ROMatch         ;find it on the bus

        ldi    temp1,ConvertTemp ;convert Temperature command
        rcall  PutByte

WaiTemp:

		  rcall  GetBit          ;wait while busy
		
		  brcc   WaiTemp         ;DS18S20 will pull bus low while busy

		  rcall ReadScratch

	     lds temp1,TLUB2
        dec temp1               ;da die MStNummern bei 1 beginnen
        clr temp2

        lsl temp1               ;Offset=MStIndex*2
        rol temp2

		  ldi ZL,low(TempMSt0)
		  ldi ZH,high(TempMSt0)		
		  		
		  add ZL,temp1		
		  adc ZH,temp2
		  lds temp1,TempMSB
		  st Z+,temp1
		  lds temp1,TempLSB
		  st Z,temp1
		
		  lds temp1,TLUB2
		  dec temp1
		  clr temp2
		  ldi ZL,low(SSTemp1)
		  ldi ZH,high(SSTemp1)
		  add ZL,temp1
		  adc ZH,temp2
		  lds temp1,TempLSB
		  lds temp2,TempMSB
		  lsr temp1
		  lsr temp1
		  lsr temp1
		  lsr temp1
		  lsl temp2
		  lsl temp2
		  lsl temp2
		  lsl temp2
		  add temp1,temp2
		  st Z,temp1		

GExit:
        rcall SetInt

        ret








